#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Git in DataFrames
# 
# python + git + dataframe = [git-pandas](http://wdm0006.github.io/git-pandas/)

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Repository

# In[3]:


from gitpandas import Repository

tries = [".", "../..", "../../.."]
err = None
for t in tries:
    try:
        repo = Repository(working_dir=t, verbose=True)
        err = None
        break
    except Exception as e:
        err = e
        continue
if err is not None:
    import os
    raise Exception("issue in current folder '{0}'".format(os.getcwd())) from err


# In[4]:


repo.branches()


# One funny function. No idea if that gives a good estimation.

# In[5]:


try:
    use = repo.hours_estimate()
except Exception as e:
    # Not always reliable.
    print(e)
    use = None
use


# In[6]:


if use is not None:
    workdays = use.hours.sum() / 8
else:
    workdays = None
workdays


# Not sure what this number reflects.

# ## Logs
# 
# The following cane take some time depending on you repository size.

# In[7]:


try:
    hist = repo.commit_history()
except Exception as e:
    # Not always reliable.
    print(e)
    import pandas
    hist = pandas.DataFrame()
hist.head()


# In[8]:


try:
    histf = repo.file_change_history()
except Exception as e:
    # Not always reliable.
    print(e)
    import pandas
    histf = pandas.DataFrame(dict(filename=[""]))
histf.head()


# In[9]:


histf.tail()


# ## Check removed files

# In[10]:


unique = set(histf.filename)
len(unique)


# In[11]:


import os
sorted_unique = list(sorted(unique))
full_path = [os.path.join(repo.repo.working_dir, _) for _ in sorted_unique]


# In[12]:


import numpy
exists = [os.path.exists(f) for f in full_path]
sizes = [os.stat(f).st_size if os.path.exists(f) else numpy.nan for f in full_path]


# In[13]:


import pandas
removed = pandas.DataFrame(dict(name=sorted_unique, exists=exists, size=sizes))
removed.sort_values("size").dropna().tail()


# How many files not exist anymore?

# In[14]:


removed[~removed.exists].shape


# In[15]:


removed[~removed.exists].head()


# In[16]:


g = repo.repo.git()


# In[17]:


print(g.execute('git log --log-size --abbrev --follow "build_script.bat"').replace(
    '@gmail.com', '@').replace("@ensae.fr", "@"))


# In[18]:


from pyquickhelper.loghelper.repositories.pygit_helper import get_repo_log
res = get_repo_log(repo.repo.working_dir)
res[0]


# In[19]:


df = pandas.DataFrame(data=res,
                      columns="owner hash datetime comment full_hash path".split())
df.head()


# In[20]:


res = get_repo_log(repo.repo.working_dir, file_detail=True)
res[0]


# In[21]:


df = pandas.DataFrame(data=res,
                      columns="owner hash datetime comment full_hash path name net bytes".split())
df.head()


# In[22]:


df["ext"] = df.name.apply(lambda x: os.path.splitext(x)[-1].strip())
df.head(n=2)


# In[23]:


gr = df[df.ext.isin((".py", ".ipynb", ".txt", ".zip", ".yml"))].groupby("ext").sum()
gr.T


# In[24]:


gr.plot(kind="bar");


# In[25]: